<?php
namespace Ep\Woo\Product;

/**
 * Class ProductTrait
 *
 * @package Ep\Woo\Product
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
trait ProductTrait
{
    /**
     * Vraća id ovog proizvoda.
     *
     * @return int
     */
    public function getId(): int
    {
        return $this->get_id();
    }

    /**
     * Najveća količina u kojoj je moguće kupiti ovaj proizvod.
     *
     * @return int
     */
    public function getMaxPurchaseQuantity(): int
    {
        return intval($this->get_max_purchase_quantity());
    }

    /**
     * Najmanja količina u kojoj je moguće kupiti ovaj proizvod.
     *
     * @return int
     */
    public function getMinPurchaseQuantity(): int
    {
        return $this->get_min_purchase_quantity();
    }

    /**
     * Naziv ovog proizvoda.
     *
     * @return string
     */
    public function getName(): string
    {
        return $this->get_name(null);
    }

    /**
     * Vraća true ako je ovaj proizvod na akciji.
     *
     * @return bool
     */
    public function isOnSale(): bool
    {
        return $this->is_on_sale(null);
    }

    /**
     * Vraća cijenu ovog proizvoda.
     *
     * @return float
     */
    public function getPrice(): float
    {
        return floatval($this->get_price(null));
    }

    /**
     * Da li je ovaj proizvod moguće kupiti.
     *
     * @return bool
     */
    public function isPurchasable(): bool
    {
        return $this->is_purchasable();
    }

    /**
     * Vraća korake u kojima je moguće kupiti proizvod.
     *
     * @return int
     */
    public function getPurchaseStep(): int
    {
        return $this->get_purchase_step();
    }

    /**
     * Vraća korake u kojima je moguće naručiti proizvod.
     */
    public function get_purchase_step()
    {
        return intval($this->get_prop('purchase_step', null)) ?: 1;
    }

    /**
     * Vraća redovnu cijenu ako je proizvod na akciji.
     *
     * @return float
     */
    public function getRegularPrice(): float
    {
        return floatval($this->get_regular_price(null));
    }

    /**
     * Vraća akcijsku cijenu proizvoda.
     *
     * @return float
     */
    public function getSalePrice(): float
    {
        return floatval($this->get_sale_price(null));
    }

    /**
     * Vraća šifru ovog proizvoda.
     *
     * @return string
     */
    public function getSku(): string
    {
        return $this->get_sku(null);
    }

    /**
     * Vraća stanje zalihe ovog proizvoda.
     *
     * @return int
     */
    public function getStock(): int
    {
        $stock = $this->get_stock_quantity(null);
        return $stock < $this->getPurchaseStep() ? 0 : $stock;
    }

    /**
     * Vraća vrstu ovog proizvoda.
     *
     * @return string
     */
    public function getType(): string
    {
        return $this->get_type();
    }

    /**
     * Vraća url proizvoda.
     *
     * @return string
     */
    public function getUrl(): string
    {
        return get_permalink($this->get_id());
    }

    /**
     * Da li je ovaj proizvod vidljiv u katalogu.
     *
     * @return bool
     */
    public function isVisible(): bool
    {
        return $this->is_visible();
    }
}