<?php
namespace Ep\Woo\View;

use Ep\Wp\View\AbstractView as BaseAbstractView;

/**
 * Osnovna klasa za WooCommerce viewove.
 *
 * @package Mishe\View
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
abstract class AbstractView extends BaseAbstractView
{
    /** @var bool */
    static $assetsAdded = false;
    /** @var array */
    protected $templateOverrides = [];
    /** @var array */
    protected $templatePartOverrides = [];

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function init(): void
    {
        parent::init();

        add_filter('wc_get_template', [$this, 'wc_get_template'], PHP_INT_MAX, 2);
        add_filter('wc_get_template_part', [$this, 'wc_get_template_part'], PHP_INT_MAX, 3);
        add_filter('woocommerce_locate_template', [$this, 'wc_locate_template'], 10, 3);

        add_action('woocommerce_before_template_part', [$this, 'wc_before_template_part'], -1, 4);

        if (false === self::$assetsAdded) {
            add_action('wp_enqueue_scripts', [$this, 'wp_enqueue_scripts'], 100);
            self::$assetsAdded = true;
        }
    }

    /**
     * Dodaje front-end resurse.
     */
    public function wp_enqueue_scripts()
    {
        wp_deregister_script('wc-add-to-cart');
        wp_dequeue_script('wc-add-to-cart');
        wp_deregister_script('wc-add-to-cart-variation');
        wp_enqueue_script('wp-util');
    }

    /**
     * Okida se prije nego što će WooCommerce učitati template.
     *
     * @param string $located
     * @param string $template_name
     *
     * @return string
     */
    public function wc_get_template(string $located, string $template_name): string
    {
        if (isset($this->templateOverrides[$template_name]) || in_array($template_name, $this->templateOverrides)) {
            return __DIR__ . '/_blank.php';
        }

        return $located;
    }

    /**
     * @param string $located
     * @param string $slug
     * @param string $template_name
     *
     * @return string
     */
    public function wc_get_template_part(string $located, string $slug, string $template_name = null): string
    {
        $name = rtrim($slug . '-' . $template_name, '-') . '.php';

        if (in_array($name, $this->templatePartOverrides) || isset($this->templatePartOverrides[$name])) {
            $this->wc_before_template_part($name);
            return __DIR__ . '/_blank.php';
        }

        return $located;
    }

    /**
     * Renderira zamijenjeni template.
     *
     * @param string $template_name
     */
    public function wc_before_template_part(string $template_name): void
    {
        $callback = $this->getCallback($template_name);

        if (is_callable($callback)) {
            call_user_func_array( $callback, func_get_args() );
        }
    }

    /**
     * @param string $template
     * @param string $template_name
     * @param string $template_path
     *
     * @return string
     */
    public function wc_locate_template(string $template, string $template_name, string $template_path): string
    {
        if (in_array($template_name, $this->templateOverrides) || isset($this->templateOverrides[$template_name])) {
            return __DIR__ . '/_blank.php';
        }

        return $template;
    }

    private function getCallback(string $template)
    {
        $callback = null;

        if (in_array($template, $this->templateOverrides)) {
            $callback = 'output';
        } else if (isset($this->templateOverrides[$template])) {
            $callback = $this->templateOverrides[$template];
        }

        if (in_array($template, $this->templatePartOverrides)) {
            $callback = 'output';
        } else if (isset($this->templatePartOverrides[$template])) {
            $callback = $this->templatePartOverrides[$template];
        }

        if (is_string($callback) && !is_callable($callback) && method_exists($this, $callback)) {
            $callback = [$this, $callback];
        }

        return $callback;
    }
}